from bluer_objects.README.items import ImageItems
from bluer_ugv.README.consts import (
    assets2_bluer_swallow,
    bluer_swallow_electrical_design,
)
from bluer_ugv.parts.db import db_of_parts
from bluer_ugv.swallow.parts import dict_of_parts
from bluer_ugv.README.swallow.digital.design.mechanical import items as mechanical_items
from bluer_ugv.README.swallow.digital.design.ultrasonic_sensor import (
    items as ultrasonic_sensor_items,
)


items = (
    [
        {
            "path": "../docs/bluer_swallow/digital/design",
        },
        {
            "path": "../docs/bluer_swallow/digital/design/operation.md",
            "items": ImageItems(
                {
                    f"{assets2_bluer_swallow}/20250915_111435.jpg": "",
                }
            ),
        },
        {
            "path": "../docs/bluer_swallow/digital/design/parts.md",
            "items": db_of_parts.as_images(
                dict_of_parts,
                reference="../../../parts",
            ),
            "macros": {
                "parts:::": db_of_parts.as_list(
                    dict_of_parts,
                    reference="../../../parts",
                    log=False,
                ),
            },
        },
        {
            "path": "../docs/bluer_swallow/digital/design/terraform.md",
            "items": ImageItems(
                {
                    f"{assets2_bluer_swallow}/20250611_100917.jpg": "",
                    f"{assets2_bluer_swallow}/lab.png": "",
                    f"{assets2_bluer_swallow}/lab2.png": "",
                }
            ),
        },
        {
            "path": "../docs/bluer_swallow/digital/design/steering-over-current-detection.md",
            "items": ImageItems(
                {
                    f"{bluer_swallow_electrical_design}/steering-over-current.png": f"{bluer_swallow_electrical_design}/steering-over-current.svg",
                }
            ),
        },
        {
            "path": "../docs/bluer_swallow/digital/design/rpi-pinout.md",
        },
    ]
    + mechanical_items
    + ultrasonic_sensor_items
)



from bluer_objects.README.items import ImageItems
from bluer_ugv.README.consts import algo_docs, assets2_bluer_swallow
from bluer_ugv.README.swallow.digital.algo.navigation import items as navigation_items
from bluer_ugv.README.swallow.digital.algo.yolo import items as yolo_items

items = (
    [
        {
            "path": "../docs/bluer_swallow/digital/algo",
        },
        {
            "path": "../docs/bluer_swallow/digital/algo/driving.md",
        },
    ]
    + navigation_items
    + [
        {
            "path": "../docs/bluer_swallow/digital/algo/tracking",
            "items": ImageItems(
                {
                    f"{assets2_bluer_swallow}/target-selection.png": f"{algo_docs}/socket.md",
                }
            ),
        }
    ]
    + yolo_items
)



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import (
    assets,
    assets2_bluer_swallow,
    bluer_swallow_mechanical_design,
)

items = ImageItems(
    {
        f"{assets}/bluer-ugv/bluer-light.png": "",
        f"{bluer_swallow_mechanical_design}/v0/3d-design.png": f"{bluer_swallow_mechanical_design}/v0/3d-design.stl",
        f"{assets2_bluer_swallow}/20250605_180136.jpg": "",
        f"{assets2_bluer_swallow}/20250608_144453.jpg": "",
        f"{assets2_bluer_swallow}/20250616_134654.jpg": "",
        f"{assets2_bluer_swallow}/20250616_145049.jpg": "",
        f"{assets2_bluer_swallow}/20250618_102816~2_1.gif": "",
        f"{assets2_bluer_swallow}/20250618_122604.jpg": "",
        f"{bluer_swallow_mechanical_design}/v0/cover.png": f"{bluer_swallow_mechanical_design}/v0/cover.stl",
        f"{assets2_bluer_swallow}/20250629_123616.jpg": "",
        f"{assets2_bluer_swallow}/20250630_214923.jpg": "",
        f"{assets2_bluer_swallow}/20250701_2206342_1.gif": "",
        f"{assets2_bluer_swallow}/20250707_122000.jpg": "",
        f"{assets2_bluer_swallow}/20250707_182818.jpg": "",
        f"{assets2_bluer_swallow}/20250709_111955.jpg": "",
        f"{assets2_bluer_swallow}/20250904_174526.jpg": "",
        f"{assets2_bluer_swallow}/20250912_211652.jpg": "",
        f"{assets2_bluer_swallow}/20250913_111410.jpg": "",
        f"{assets2_bluer_swallow}/20250913_200401.jpg": "",
        f"{assets2_bluer_swallow}/20250913_203635~2_1.gif": "",
    }
)



from bluer_ugv.README.swallow.digital.algo import items as algo_items
from bluer_ugv.README.swallow.digital.design import items as design_items

items = (
    [
        {
            "path": "../docs/bluer_swallow/digital",
        }
    ]
    + design_items
    + algo_items
)



github_kamangir = "https://github.com/kamangir"

algo_docs = f"{github_kamangir}/bluer-algo/blob/main/bluer_algo/docs"

assets = f"{github_kamangir}/assets/raw/main"

assets2 = f"{github_kamangir}/assets2/raw/main"

assets2_bluer_eagle = f"{assets2}/bluer-eagle"
assets2_bluer_fire = f"{assets2}/bluer-fire"
assets2_bluer_robin = f"{assets2}/bluer-robin"
assets2_bluer_swallow = f"{assets2}/bluer-swallow"
assets2_bluer_sparrow = f"{assets2}/bluer-sparrow"
assets2_bluer_ugv = f"{assets2}/bluer-ugv"

# design
bluer_ugv_mechanical_design = f"{github_kamangir}/bluer-ugv-mechanical-design/"

bluer_robin_design = f"{bluer_ugv_mechanical_design}/blob/main/bluer_robin"

bluer_swallow_electrical_design = (
    f"{bluer_ugv_mechanical_design}/blob/main/bluer_swallow/electrical"
)
bluer_swallow_mechanical_design = (
    f"{bluer_ugv_mechanical_design}/blob/main/bluer_swallow/mechanical"
)
bluer_swallow_ultrasonic_sensor_design = (
    f"{bluer_ugv_mechanical_design}/blob/main/bluer_swallow/ultrasonic-sensors"
)

bluer_sparrow_mechanical_design = (
    f"{bluer_ugv_mechanical_design}/blob/main/bluer_sparrow"
)



from blueness import module

from bluer_ugv import NAME
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.log import (
    UltrasonicSensorDetectionLog,
)
from bluer_ugv.logger import logger


NAME = module.name(__file__, NAME)


def review(
    object_name: str,
    export_gif: bool = False,
    log: bool = True,
) -> bool:
    logger.info("{}.review({})".format(NAME, object_name))

    detection_log = UltrasonicSensorDetectionLog()

    if not detection_log.load(
        object_name=object_name,
    ):
        return False

    if not detection_log.export(
        object_name=object_name,
        line_width=80,
        export_gif=export_gif,
        log=log,
    ):
        return False

    return True



from bluer_ugv.swallow.session.classical.leds import ClassicalLeds
from bluer_ugv.swallow.session.classical.motor.generic import GenericMotor
from bluer_ugv.swallow.session.classical.setpoint.classes import ClassicalSetPoint


class ClassicalRearMotors(GenericMotor):
    def __init__(
        self,
        setpoint: ClassicalSetPoint,
        leds: ClassicalLeds,
    ):
        super().__init__(
            role="speed",
            lpwm_pin=19,
            rpwm_pin=13,
            setpoint=setpoint,
            leds=leds,
        )



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import (
    assets2,
    assets2_bluer_swallow,
)

items = [
    {
        "path": "../docs/bluer_swallow/digital/design/ultrasonic-sensor",
    },
    {
        "path": "../docs/bluer_swallow/digital/design/ultrasonic-sensor/dev.md",
    },
    {
        "path": "../docs/bluer_swallow/digital/design/ultrasonic-sensor/tester.md",
        "cols": 2,
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/20250918_122725.jpg": "",
                f"{assets2_bluer_swallow}/20250918_194715-2.jpg": "",
                f"{assets2_bluer_swallow}/20250918_194804_1.gif": "",
                f"{assets2}/ultrasonic-sensor-tester/00.jpg": "",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/design/ultrasonic-sensor/shield.md",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/20250923_142200.jpg": "",
                f"{assets2_bluer_swallow}/20250923_145111.jpg": "",
            }
        ),
    },
]



from bluer_objects.README.items import ImageItems
from bluer_ugv.README.consts import assets, assets2_bluer_swallow

items = [
    {
        "path": "../docs/bluer_swallow/digital/algo/yolo",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/yolo-debug.png": "",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/yolo/train.md",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/yolo-debug-{0}.png": "",
                f"{assets2_bluer_swallow}/yolo-debug-{2}.png": "",
                f"{assets2_bluer_swallow}/yolo-debug-{3}.png": "",
                f"{assets2_bluer_swallow}/yolo-debug-{4}.png": "",
                f"{assets2_bluer_swallow}/yolo-debug-{5}.png": "",
                f"{assets}/swallow-debug-2025-09-16-19-53-19-4yzsp8/swallow-debug-2025-09-16-19-53-19-4yzsp8-2.gif": "",
            }
        ),
    },
]



from bluer_objects.README.items import ImageItems
from bluer_ugv.README.consts import algo_docs, assets, assets2_bluer_swallow

items = [
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/dataset",
        "items": ImageItems(
            {
                f"{assets}/swallow-dataset-2025-07-11-10-53-04-n3oybs/grid.png": "./digital/dataset/combination/validation.md",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/dataset/collection",
        "items": ImageItems(
            {
                f"{assets2_bluer_swallow}/2025-07-08-13-09-38-so54ao.png": "",
                f"{assets2_bluer_swallow}/2025-07-09-11-20-27-4qf255-000-2.png": "",
                f"{assets2_bluer_swallow}/2025-07-09-11-18-07-azy27w.png": f"{algo_docs}/image_classifier/dataset/sequence.md",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/dataset/collection/validation.md"
    },
    {"path": "../docs/bluer_swallow/digital/algo/navigation/dataset/collection/one.md"},
    {"path": "../docs/bluer_swallow/digital/algo/navigation/dataset/combination"},
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/dataset/combination/validation.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/dataset/combination/one.md"
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/dataset/review.md",
    },
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/model",
    },
    {"path": "../docs/bluer_swallow/digital/algo/navigation/model/validation.md"},
    {
        "path": "../docs/bluer_swallow/digital/algo/navigation/model/one.md",
    },
]



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_debug(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~upload", mono=mono)

    args = [
        "[--generate_gif 0]",
        "[--save_images 0]",
    ]

    return show_usage(
        [
            "@swallow",
            "debug",
            f"[{options}]",
            "[-|<object-name>]",
        ]
        + args,
        "debug swallow.",
        mono=mono,
    )



from typing import List
import numpy as np

from bluer_options.timer import Timer
from bluer_options import string
from bluer_options import host
from bluer_objects.storage.policies import DownloadPolicy
from bluer_objects import storage
from bluer_objects.metadata import post_to_object, get_from_object
from bluer_sbc.imager.camera import instance as camera
from bluer_algo.image_classifier.dataset.dataset import ImageClassifierDataset
from bluer_algo.image_classifier.model.predictor import ImageClassifierPredictor

from bluer_ugv import env
from bluer_ugv.swallow.session.classical.camera.generic import ClassicalCamera
from bluer_ugv.swallow.session.classical.keyboard import ClassicalKeyboard
from bluer_ugv.swallow.session.classical.leds import ClassicalLeds
from bluer_ugv.swallow.session.classical.setpoint.classes import ClassicalSetPoint
from bluer_ugv.swallow.session.classical.mode import OperationMode
from bluer_ugv.logger import logger


class ClassicalNavigationCamera(ClassicalCamera):
    def __init__(
        self,
        keyboard: ClassicalKeyboard,
        leds: ClassicalLeds,
        setpoint: ClassicalSetPoint,
        object_name: str,
    ):
        super().__init__(keyboard, leds, setpoint, object_name)

        self.prediction_timer = Timer(
            period=env.BLUER_UGV_CAMERA_ACTION_PERIOD,
            name="{}.prediction".format(self.__class__.__name__),
            log=True,
        )
        self.training_timer = Timer(
            period=env.BLUER_UGV_CAMERA_TRAINING_PERIOD,
            name="{}.training".format(self.__class__.__name__),
            log=True,
        )

        self.dict_of_classes = {
            0: "no_action",
            1: "left",
            2: "right",
        }

        self.dataset = ImageClassifierDataset(
            dict_of_classes=self.dict_of_classes,
            object_name=self.object_name,
        )

        self.predictor = None

        self.buffer_size = -1
        self.buffer: List[np.ndarray] = []

    def initialize(self) -> bool:
        if not super().initialize():
            return False

        if not storage.download(
            env.BLUER_UGV_SWALLOW_NAVIGATION_MODEL,
            policy=DownloadPolicy.DOESNT_EXIST,
        ):
            return False

        success, self.predictor = ImageClassifierPredictor.load(
            object_name=env.BLUER_UGV_SWALLOW_NAVIGATION_MODEL,
        )
        if not success:
            return success

        if self.predictor.shape[0] != camera.resolution[0]:
            logger.error(
                "height mismatch: {} <> {}".format(
                    self.predictor.shape[0],
                    camera.resolution[0],
                )
            )
            return False

        buffer_size = self.predictor.shape[1] / camera.resolution[1]
        if int(buffer_size) != buffer_size:
            logger.error(
                "non-integer buffer size: {} / {} = {:.2f}".format(
                    self.predictor.shape[1], camera.resolution[1], buffer_size
                )
            )
            return False
        self.buffer_size = int(buffer_size)
        logger.info(f"buffer size: {self.buffer_size}")

        return True

    def cleanup(self):
        super().cleanup()

        self.dataset.save(
            metadata={
                "source": host.get_name(),
            },
            log=True,
        )

        if self.dataset.df.empty:
            return

        dataset_list: List[str] = get_from_object(
            object_name=env.BLUER_UGV_SWALLOW_NAVIGATION_DATASET_LIST,
            key="dataset-list",
            default=[],
            download=True,
        )
        dataset_list.append(self.object_name)
        if not post_to_object(
            object_name=env.BLUER_UGV_SWALLOW_NAVIGATION_DATASET_LIST,
            key="dataset-list",
            value=dataset_list,
            upload=True,
            verbose=True,
        ):
            logger.error("failed to add object to dataset list.")

    def update(self) -> bool:
        if not super().update():
            return False

        if self.setpoint.speed <= 0:
            self.buffer = []
            return True

        if self.keyboard.mode == OperationMode.ACTION:
            return self.update_action()

        if self.keyboard.mode == OperationMode.TRAINING:
            return self.update_training()

        return True

    def update_action(self) -> bool:
        if not self.prediction_timer.tick():
            return True

        self.leds.leds["red"]["state"] = not self.leds.leds["red"]["state"]

        success, image = camera.capture(
            close_after=False,
            open_before=False,
            log=True,
        )
        if not success:
            return success

        self.buffer.append(image)
        if len(self.buffer) > self.buffer_size:
            self.buffer = self.buffer[1:]
        if len(self.buffer) < self.buffer_size:
            logger.info("buffering ...")
            return True
        if len(self.buffer) > self.buffer_size:
            logger.error("buffer overflow - this must not happen.")
            return False

        success, metadata = self.predictor.predict(
            image=np.hstack(self.buffer),
        )
        if not success:
            return success

        predicted_class = metadata["predicted_class"]
        if predicted_class == 1:
            self.setpoint.put(
                what="steering",
                value=env.BLUER_UGV_SWALLOW_STEERING_SETPOINT,
                log=True,
            )
        elif predicted_class == 2:
            self.setpoint.put(
                what="steering",
                value=-env.BLUER_UGV_SWALLOW_STEERING_SETPOINT,
                log=True,
            )

        return True

    def update_training(self) -> bool:
        if not (self.training_timer.tick() or self.keyboard.last_key != ""):
            return True

        self.leds.leds["red"]["state"] = not self.leds.leds["red"]["state"]

        filename = "{}.png".format(
            string.pretty_date(
                as_filename=True,
                unique=True,
            )
        )

        success, _ = camera.capture(
            close_after=False,
            open_before=False,
            object_name=self.object_name,
            filename=filename,
            log=True,
        )
        if not success:
            return success

        logger.info(f"self.keyboard.last_key: {self.keyboard.last_key}")

        if not self.dataset.add(
            filename=filename,
            class_index=(
                0
                if self.keyboard.last_key == ""
                else 1 if self.keyboard.last_key == "a" else 2
            ),
            log=True,
        ):
            return False

        self.training_timer.reset()

        return True



from bluer_objects.README.items import ImageItems, Items
from bluer_objects import markdown

from bluer_ugv.README.consts import (
    assets2_bluer_swallow,
    assets2_bluer_ugv,
    bluer_swallow_electrical_design,
)

items = markdown.generate_table(
    Items(
        [
            {
                "name": "parts",
                "url": "./bluer_ugv/docs/bluer_swallow/digital/design/parts.md",
                "marquee": f"{assets2_bluer_ugv}/XL4015.png",
            },
            {
                "name": "shield",
                "url": "./bluer_ugv/docs/bluer_swallow/digital/design/shield.md",
                "marquee": f"{bluer_swallow_electrical_design}/digital.png?raw=true",
            },
            {
                "name": "terraform",
                "url": "./bluer_ugv/docs/bluer_swallow/digital/design/terraform.md",
                "marquee": f"{assets2_bluer_swallow}/20250611_100917.jpg?raw=true",
            },
        ]
    )
)



from typing import Dict
import numpy as np


class Detection:
    def __init__(
        self,
        side: str,
        detection: bool = False,
        reason: str = "",
        echo_detected: bool = False,
        pulse_ms: float = 0.0,
        distance_mm: float = 0.0,
    ) -> None:
        self.side = side

        self.detection = detection
        self.reason = reason

        self.echo_detected = echo_detected
        self.pulse_ms = pulse_ms
        self.distance_mm = distance_mm

    def as_image(
        self,
        height: int = 512,
        width: int = 256,
        max_m: float = 0.8,
    ) -> np.ndarray:
        image = np.zeros((height, width, 3), dtype=np.uint8)

        if not self.detection:
            image[:, :width, 0] = 255
        elif not self.echo_detected:
            image[:, :width, :2] = 255
        else:
            distance = max(
                min(
                    int(self.distance_mm / 1000 / max_m * height),
                    height,
                ),
                0,
            )
            image[height - distance :, :, :] = 128

        return image

    def as_dict(self) -> Dict:
        return {
            "side": self.side,
            "detection": self.detection,
            "reason": self.reason,
            "echo_detected": self.echo_detected,
            "pulse_ms": self.pulse_ms,
            "distance_mm": self.distance_mm,
        }

    def as_str(
        self,
        short: bool = False,
    ) -> str:
        if self.detection:
            return ("{}: {}" if short else "{:8}: {}").format(
                self.side,
                (
                    (
                        "{:.2f} ms == {:.0f} mm"
                        if short
                        else "{:6.2f} ms == {:5.0f} mm"
                    ).format(
                        self.pulse_ms,
                        self.distance_mm,
                    )
                    if self.echo_detected
                    else "no echo" if self.detection else "no detection"
                ),
            )

        return f"{self.side}: no detection ({self.reason})"



from typing import Any

from bluer_algo.socket.classes import SocketComm, DEV_HOST
from bluer_sbc.imager.camera import instance as camera
from bluer_sbc.env import BLUER_SBC_ENV

from bluer_ugv.swallow.session.classical.keyboard import ClassicalKeyboard
from bluer_ugv.swallow.session.classical.leds import ClassicalLeds
from bluer_ugv.swallow.session.classical.setpoint.classes import ClassicalSetPoint
from bluer_ugv.logger import logger


class ClassicalCamera:
    def __init__(
        self,
        keyboard: ClassicalKeyboard,
        leds: ClassicalLeds,
        setpoint: ClassicalSetPoint,
        object_name: str,
    ):
        logger.info(f"sbc.env: {BLUER_SBC_ENV}")

        self.keyboard = keyboard
        self.leds = leds
        self.setpoint = setpoint

        self.object_name = object_name

    def initialize(self) -> bool:
        return camera.open(log=True)

    def cleanup(self):
        camera.close(log=True)

    def send_debug_data(self, data: Any) -> bool:
        socket = SocketComm.connect_to(DEV_HOST)
        return socket.send_data(data)

    def update(self) -> bool:
        return True



import os
from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)


BLUER_BEAST_MODEL = get_env("BLUER_BEAST_MODEL")

BLUER_UGV_CAMERA_TRAINING_PERIOD = get_env("BLUER_UGV_CAMERA_TRAINING_PERIOD", 3.0)
BLUER_UGV_CAMERA_ACTION_PERIOD = get_env("BLUER_UGV_CAMERA_ACTION_PERIOD", 1.5)

BLUER_UGV_SWALLOW_STEERING_SETPOINT = get_env("BLUER_UGV_SWALLOW_STEERING_SETPOINT", 50)

BLUER_UGV_MOUSEPAD_ENABLED = get_env("BLUER_UGV_MOUSEPAD_ENABLED", 0)

BLUER_UGV_SWALLOW_NAVIGATION_DATASET_LIST = get_env(
    "BLUER_UGV_SWALLOW_NAVIGATION_DATASET_LIST"
)

BLUER_UGV_SWALLOW_NAVIGATION_MODEL = get_env("BLUER_UGV_SWALLOW_NAVIGATION_MODEL", "")

BLUER_UGV_SWALLOW_YOLO_MODEL = get_env("BLUER_UGV_SWALLOW_YOLO_MODEL")

BLUER_UGV_SWALLOW_YOLO_DATASET_LIST = get_env("BLUER_UGV_SWALLOW_YOLO_DATASET_LIST")

BLUER_UGV_ULTRASONIC_SENSOR_TEST_OBJECT = get_env(
    "BLUER_UGV_ULTRASONIC_SENSOR_TEST_OBJECT"
)



from typing import List
import matplotlib.pyplot as plt
from tqdm import tqdm
import numpy as np
from functools import reduce

from bluer_objects.graphics.signature import justify_text
from bluer_objects import objects
from bluer_objects import file, path
from bluer_objects.graphics.gif import generate_animated_gif
from bluer_objects.graphics.signature import add_signature

from bluer_ugv.logger import logger
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.detection import (
    Detection,
)
from bluer_ugv.host import signature


class UltrasonicSensorDetectionLog:
    def __init__(self):
        self.log: List[List[Detection]] = []

    def append(self, detection: List[Detection]):
        self.log.append(detection)

    def export(
        self,
        object_name: str,
        export_gif: bool = False,
        line_width: int = 80,
        log: bool = True,
    ) -> bool:
        for func, name in zip(
            [
                lambda detection: int(detection.detection),
                lambda detection: int(detection.echo_detected),
                lambda detection: detection.pulse_ms,
                lambda detection: detection.distance_mm,
            ],
            [
                "detection",
                "echo detection",
                "pulse (ms)",
                "distance(mm)",
            ],
        ):
            plt.figure(figsize=(5, 5))
            plt.plot(
                [func(list_of_detections[0]) for list_of_detections in self.log],
                color="green",
            )
            plt.plot(
                [func(list_of_detections[1]) for list_of_detections in self.log],
                color="blue",
            )

            plt.title(
                justify_text(
                    " | ".join(
                        [
                            "ultrasonic-sensor",
                        ]
                        + objects.signature(object_name=object_name)
                    ),
                    line_width=line_width,
                    return_str=True,
                )
            )
            plt.xlabel(
                justify_text(
                    " | ".join(signature()),
                    line_width=line_width,
                    return_str=True,
                )
            )
            plt.ylabel(name)
            plt.legend(["left", "right"])
            plt.tight_layout()
            plt.grid(True)
            if not file.save_fig(
                objects.path_of(
                    object_name=object_name,
                    filename="{}.png".format(
                        name.replace(" ", "-").replace("(", "-").replace(")", "-")
                    ),
                ),
                log=log,
            ):
                return False

        if export_gif:
            max_m = (
                max(
                    detection.distance_mm
                    for detection in reduce(lambda x, y: x + y, self.log, [])
                )
                / 1000
            )

            logger.info("max distance: {:.3f} m".format(max_m))

            if not self.export_gif(
                object_name=object_name,
                line_width=line_width,
                max_m=max_m,
                log=log,
            ):
                return False

        return True

    def export_gif(
        self,
        object_name: str,
        line_width: int = 80,
        height: int = 512,
        width: int = 512,
        max_m: float = 0.8,
        log: bool = True,
    ) -> bool:
        image_list: List[str] = []

        temp_folder = objects.path_of(
            object_name=object_name,
            filename="frames",
        )
        if not path.create(temp_folder):
            return False

        for index, detections in tqdm(enumerate(self.log)):
            filename = objects.path_of(
                object_name=object_name,
                filename=f"frames/{index:010d}.png",
            )

            image = np.concatenate(
                [
                    detection.as_image(
                        height=height,
                        width=int(width / len(detections)),
                        max_m=max_m,
                    )
                    for detection in detections
                ],
                axis=1,
            )

            image = add_signature(
                image,
                header=[
                    " | ".join(
                        ["ultrasonic-sensor"]
                        + [detection.as_str(short=True) for detection in detections]
                        + objects.signature(
                            f"frame #{index:04d}",
                            object_name,
                        )
                    )
                ],
                footer=[" | ".join(signature())],
                line_width=line_width,
            )

            if not file.save_image(filename, image):
                return False

            image_list.append(filename)

        if not generate_animated_gif(
            image_list,
            objects.path_of(
                filename="ultrasonic-sensor-detections.gif",
                object_name=object_name,
            ),
            log=log,
        ):
            return False

        return path.delete(temp_folder)

    def load(
        self,
        object_name: str,
    ) -> bool:
        success, self.log = file.load(
            objects.path_of(
                object_name=object_name,
                filename="detections.dill",
            ),
        )

        if success:
            logger.info("loaded {} detection(s).".format(len(self.log)))

        return success

    def save(
        self,
        object_name: str,
        log: bool = True,
    ) -> bool:
        if not file.save_yaml(
            objects.path_of(
                object_name=object_name,
                filename="detections.yaml",
            ),
            {
                "detections": [
                    [detection.as_dict() for detection in list_of_detections]
                    for list_of_detections in self.log
                ]
            },
            log=log,
        ):
            return False

        if not file.save(
            objects.path_of(
                object_name=object_name,
                filename="detections.dill",
            ),
            self.log,
            log=log,
        ):
            return False

        return True



dict_of_parts = {
    "rpi3bp": "",
    "dc-motor-12-VDC-45W": "type 2, 2 x right + 2 x left",
    "SLA-Battery": "",
    "XL4015": "",
    "BTS7960": "2 x",
    "470-mF": "",
    "Polyfuse": "optional",
    "TVS-diode": "",
    "resistor": "7 x 330-470 Ω + N x 2.2 kΩ + N x 3.3 kΩ",
    "LED": "green + red + yellow + 4 x blue",
    "rpi-camera": "",
    "PCB-single-14x9_5": "2 x",
    "PCB-double-9x7": "2 x",
    "pushbutton": "",
    "wheel": "4 x ",
    "ultrasonic-sensor": "4 x",
    "connector": "8 pairs",
}



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import assets2_bluer_sparrow

items = ImageItems(
    {
        f"{assets2_bluer_sparrow}/20250722_174115-2.jpg": "",
        f"{assets2_bluer_sparrow}/20250825_190410.jpg": "",
        f"{assets2_bluer_sparrow}/20250825_190439.jpg": "",
        f"{assets2_bluer_sparrow}/20250825_190450.jpg": "",
        f"{assets2_bluer_sparrow}/20250825_190512.jpg": "",
        f"{assets2_bluer_sparrow}/20250830_125608.jpg": "",
        f"{assets2_bluer_sparrow}/20250830_125643.jpg": "",
        f"{assets2_bluer_sparrow}/20250830_132019.jpg": "",
        f"{assets2_bluer_sparrow}/VID-20250830-WA0000~3_1.gif": "",
        f"{assets2_bluer_sparrow}/20250901_105247.jpg": "",
        f"{assets2_bluer_sparrow}/20250905_120625.jpg": "",
        f"{assets2_bluer_sparrow}/VID-20250905-WA0014_1.gif": "./validation/village-1.md",
    }
)



from bluer_ugv.README.consts import assets2

NAME = "bluer_ugv"

ICON = "🐬"

DESCRIPTION = f"{ICON} AI x UGV."

VERSION = "6.1031.1"

REPO_NAME = "bluer-ugv"

MARQUEE = f"{assets2}/bluer-sparrow/VID-20250905-WA0014_1.gif"


ALIAS = "@ugv"


def fullname() -> str:
    return f"{NAME}-{VERSION}"



from bluer_ugv.swallow.README import items
from bluer_ugv.README.swallow.analog import items as items_analog
from bluer_ugv.README.swallow.digital import items as items_digital

docs = (
    [
        {
            "items": items,
            "path": "../docs/bluer_swallow",
        }
    ]
    + items_analog
    + items_digital
)



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import bluer_swallow_electrical_design

items = [
    {
        "path": "../docs/bluer_swallow/analog",
        "items": ImageItems(
            {
                f"{bluer_swallow_electrical_design}/analog.png": f"{bluer_swallow_electrical_design}/analog.svg",
            }
        ),
    }
]



from blueness import module
import numpy as np
import cv2
from typing import List

from bluer_options import string
from bluer_objects import file
from bluer_objects import objects
from bluer_objects.graphics.gif import generate_animated_gif
from bluer_algo.socket.classes import SocketComm
from bluer_algo.tracker.classes.target import Target

from bluer_ugv import NAME
from bluer_ugv.logger import logger


NAME = module.name(__file__, NAME)


def debug(
    object_name: str,
    generate_gif: bool = True,
    save_images: bool = True,
) -> bool:
    logger.info(
        "{}.debug -{}{}> {}".format(
            NAME,
            "images-" if save_images else "",
            "gif-" if generate_gif else "",
            object_name,
        )
    )

    socket = SocketComm.listen_on()

    title = "debug..."

    cv2.namedWindow(title)
    logger.info("Ctrl+C to exit...")

    image = np.zeros((480, 640, 3), np.uint8)

    list_of_images: List[str] = []
    try:
        while True:
            cv2.imshow(title, np.flip(image, axis=2))
            cv2.waitKey(1)

            success, image = socket.receive_data(np.ndarray)
            if not success:
                break

            if save_images:
                filename = objects.path_of(
                    filename="{}.png".format(string.timestamp()),
                    object_name=object_name,
                )

                if not file.save_image(filename, image, log=True):
                    break

                list_of_images.append(filename)
    except KeyboardInterrupt:
        logger.info("Ctrl+C, stopping.")

    cv2.destroyWindow(title)

    if generate_gif:
        if not generate_animated_gif(
            list_of_images,
            objects.path_of(
                filename=f"{object_name}.gif",
                object_name=object_name,
            ),
        ):
            return False

    return True



# continues -v7

from typing import List

from bluer_objects import file
from bluer_objects import objects

from bluer_ugv.logger import logger
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.detection import (
    Detection,
)
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.log import (
    UltrasonicSensorDetectionLog,
)


def test(
    object_name: str,
    max_m: float = 0.8,
    export: bool = True,
    export_gif: bool = False,
    log: bool = True,
    line_width: int = 80,
) -> bool:
    from RPi import GPIO

    from bluer_ugv.swallow.session.classical.ultrasonic_sensor.pack import (
        UltrasonicSensorPack,
    )

    ultrasonic_sensor_pack = UltrasonicSensorPack(max_m=max_m)
    if not ultrasonic_sensor_pack.valid:
        return False

    detection_log = UltrasonicSensorDetectionLog()

    success = True
    try:
        while True:
            success, detection = ultrasonic_sensor_pack.detect(log=log)
            if not success:
                break

            if export:
                detection_log.append(detection)
    except KeyboardInterrupt:
        logger.info("^C detected.")
    finally:
        GPIO.cleanup()

    if not export:
        return success

    if not success:
        return success

    if not detection_log.export(
        object_name=object_name,
        line_width=line_width,
        export_gif=export_gif,
        log=log,
    ):
        return False

    return detection_log.save(object_name=object_name)



from bluer_objects.README.items import ImageItems
from bluer_ugv.README.consts import bluer_swallow_mechanical_design

items = [
    {
        "path": "../docs/bluer_swallow/digital/design/mechanical",
        "items": ImageItems(
            {
                f"{bluer_swallow_mechanical_design}/robot.png": f"{bluer_swallow_mechanical_design}/robot.stl",
                f"{bluer_swallow_mechanical_design}/cage.png": f"{bluer_swallow_mechanical_design}/cage.stl",
                f"{bluer_swallow_mechanical_design}/measurements.png": "",
            }
        ),
    },
    {
        "path": "../docs/bluer_swallow/digital/design/mechanical/v1.md",
        "items": ImageItems(
            {
                f"{bluer_swallow_mechanical_design}/v1/robot.png": f"{bluer_swallow_mechanical_design}/v1/robot.stl",
                f"{bluer_swallow_mechanical_design}/v1/cage.png": f"{bluer_swallow_mechanical_design}/v1/cage.stl",
                f"{bluer_swallow_mechanical_design}/v1/measurements.png": "",
            }
        ),
    },
]



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.consts import (
    assets2_bluer_swallow,
    bluer_swallow_electrical_design,
    bluer_swallow_ultrasonic_sensor_design,
)

items = ImageItems(
    {
        f"{bluer_swallow_electrical_design}/digital.png": f"{bluer_swallow_electrical_design}/digital.svg",
        f"{assets2_bluer_swallow}/20250609_164433.jpg": "",
        f"{assets2_bluer_swallow}/20250614_102301.jpg": "",
        f"{assets2_bluer_swallow}/20250614_114954.jpg": "",
        f"{assets2_bluer_swallow}/20250615_192339.jpg": "",
        f"{assets2_bluer_swallow}/20250703_153834.jpg": "",
        f"{assets2_bluer_swallow}/design/v2/01.jpg": "",
        f"{assets2_bluer_swallow}/design/v3/01.jpg": "",
        f"{assets2_bluer_swallow}/design/v4/01.jpg": "",
        f"{bluer_swallow_ultrasonic_sensor_design}/geometry.png?raw=true": f"{bluer_swallow_ultrasonic_sensor_design}/geometry.svg",
        f"{assets2_bluer_swallow}/20250925_213013.jpg": "",
        f"{assets2_bluer_swallow}/20250925_214017.jpg": "",
        f"{assets2_bluer_swallow}/20250928_160425.jpg": "",
        f"{assets2_bluer_swallow}/20250928_160449.jpg": "",
        f"{assets2_bluer_swallow}/design/head-v1/01.jpg": "",
    }
)


docs = [
    {
        "path": "../docs/bluer_swallow/digital/design/shield.md",
        "items": items,
    },
]
